/* =============================================================================
 UPC - EETAC - CSD - http://digsys.upc.edu 
Phase 3  
An example on how to implement a sequential system like an 1-digit BCD counter 
using "current_state" state-type variable. 
Thus, this example can be used to adapt other FSM containing a small number 
of states. 

 * Print in color this file, and analyse it and use it as 'theory' class notes. 
============================================================================= */

// This general header file includes the device specific headers like
// "pic18f4520.h". This file contains all the microcontroller declarations.
#include <xc.h>     
#include <stdio.h>
#include "lcd.h"
/* 	============================================================================
	Definitions 
   	========================================================================= */
// This structure allows to access to a given bit into an 8-bit register
#define 	PORTBIT(adr, bit) ( (unsigned)(&adr)*8 + (bit) )
// Our counter has 13 states defined as follows:
#define 	Idle 			'I' //States enumeration. "Char"
#define 	Num_0 			'0'	
#define 	Num_1 			'1'
#define 	Num_2 			'2'
#define 	Num_3 			'3'
#define 	Num_4 			'4'
#define 	Num_5 			'5'
#define 	Num_6 			'6'
#define 	Num_7 			'7'
#define 	Num_8 			'8'
#define 	Num_9 			'9'
#define 	Num_10 			'A'
#define 	Num_11 			'B'

 //Other constants:
#define 	Counter_var_Max_Count 	10		//(for 20 s )
									
#define 	Timer2_period_Reg 		250

/* 	============================================================================
	Function prototypes
   	========================================================================= */
// The three basic functions to organise the code to emulate a FSM
void init_system (void);         
char state_logic(void);
char output_logic(void);    

//The interrupt service routine (ISR) to detect the Var_CLK_Flag (external INT0)
// or the ST_SP signal (RB interrupt, or the TMR0 interrupt
static void interrupt ISR(void); 

// The auxiliary functions to read/poll and write port pins and be able to
// organise the software variables, to make the main program hardware
// independent and compatible for many microcontrollers. 
void write_outputs (void);
void read_inputs(void);  

/* 	============================================================================
	Global variables  
  	========================================================================= */
// Even if some variables are 'bit' type, we use 'char' (byte) to make them easy 
// to watch while debugging.
static char Var_CE;
static char Var_UD_L;
static char Var_TC12;
static char Var_Q;
static char Var_TMR2;
static char TMR2_wave;

//The flag that will be set from the interrupt service routine to indicate that 
// a new INT0 or Timmer has arrived or when ST_SP button is pushed or when the has to 
//write something new
static char Var_CLK_Flag;	
static char St_Sp_flag;
static char LCD_Flag;
static char Timer_Flag;
static char Var_INT0_Flag;

static int Counter_var = 0;		// The auxiliary timer variable to enlarge timing periods  
static int Count_wave=0;

static char current_state = Idle;

/* 	============================================================================
	Main function 
  	========================================================================= */
void main(void){
    init_system ();
    while(1) {                      /* loop forever the FSM */
		if (state_logic()) break;
		if (output_logic()) break;
    }
    while (1){
        __nop();//The system has crashed in an illegal state and needs attention
                //This second infinite loop is never reached in normal operation
    }           // The only way to scape is clicking reset (CD)
}	

/* 	============================================================================
	Function definitions
  	========================================================================= */

//*****************************************************************************
//State variable logic routine  (The CC1)
//*****************************************************************************
char state_logic(void){
	char error = 0;	

    read_inputs(); //sample input values so that the CC1 can decide transitions
    
    if(Var_TMR2==1){				//depending if TMR2 is on 
        Var_CLK_Flag = Timer_Flag;  //the CLK flag will be the 
    }else{							//INT0 or the Timer
         Var_CLK_Flag = Var_INT0_Flag;
    }
    
	if (Var_CLK_Flag == 1){

    	switch (current_state) {
            case Idle:
                if((Var_CE == 1) && (Var_UD_L == 0) && (St_Sp_flag == 1)){ // if the ST_SP has been pushed 
                    current_state = Num_11;								   //then it will perimit to change state
                    St_Sp_flag=0;	//reset flag after operation		   //depending in the direction (up or down)
                    LCD_Flag=1;		//due to change of state we right somthing new in the lcd
                }
                else if((Var_CE == 1) && (Var_UD_L == 1)&& (St_Sp_flag == 1)) {
                    current_state = Num_0;
                    St_Sp_flag=0;	
                    LCD_Flag=1;		
                }else{
                    current_state = Idle;
                }                
            break;
            case Num_0:
                if((Var_CE == 1) && (Var_UD_L == 0) && (St_Sp_flag == 1)){
                    current_state = Idle;
                    St_Sp_flag=0;	
                    LCD_Flag=1;		
                }
                else if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_1;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0) && (St_Sp_flag == 0)) {
                    current_state = Num_11;
                    LCD_Flag=1;		
                    
                }else{
                    current_state = Num_0;
                }                    
            break;

            case Num_1:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_2;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_0;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_1;
                }                    
            break;
            
            case Num_2:		
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_3;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_1;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_2;
                }                         
            break;

            case Num_3:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_4;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_2;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_3;
                }    
            break;
            case Num_4:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_5;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_3;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_4;
                }    
            break;
            case Num_5:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_6;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_4;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_5;
                }    
            break;
            case Num_6:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_7;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_5;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_6;
                }    
            break;
            case Num_7:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_8;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_6;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_7;
                }    
            break;
            case Num_8:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_9;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_7;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_8;
                }    
            break;
            case Num_9:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_10;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_8;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_9;
                }    
            break;
            case Num_10:				
                if ((Var_CE == 1) && (Var_UD_L == 1)){
                    current_state = Num_11;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_9;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_10;
                }    
            break;
            case Num_11:				
                 if((Var_CE == 1) && (Var_UD_L == 1) && (St_Sp_flag == 1)){
                    current_state = Idle;
                    St_Sp_flag=0;	
                    LCD_Flag=1;		
                }
                 else if ((Var_CE == 1) && (Var_UD_L == 1) && (St_Sp_flag == 0)){
                    current_state = Num_0;
                    LCD_Flag=1;		
                }
                else if((Var_CE == 1) && (Var_UD_L == 0)) {
                    current_state = Num_10;
                    LCD_Flag=1;		
                }else{
                    current_state = Num_11;
                }    
            break;

            default:
            // If no states have been selected, there is an error
            // and the systems must be stopped                     
                error = 1;	
                break;
        }
        Var_INT0_Flag=0;	// Clear the Var_CLK_Flag detection flag
        Timer_Flag=0;
	}
	return (error);
}

//*****************************************************************************
//Output logic routine (The combinational circuit CC2)
//*****************************************************************************
char output_logic(void){
unsigned char error = 0;	 
	switch (current_state) {
        case Idle:
            Var_Q = 0;//idle	
            Var_TC12 = 0;
             if (LCD_Flag == 1) {			//write only if flag activates	
                lcd_clear();
                lcd_home();						// select line 1
                lcd_puts("IDLE mode ...");
                lcd_home2();			// select line 2 to write the result
                lcd_puts(" ... push ST_SP ...");
                }
            LCD_Flag = 0;
            break;
		case Num_0:
            Var_Q = 0;//0
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {	//depending if counting up`or down 			
                lcd_clear();						//the lcd will give a different output
                lcd_home();						
                lcd_puts("0 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("0 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_1:
            Var_Q = 0b00100000;//1
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {				
                lcd_clear();
                lcd_home();						
                lcd_puts("1 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("1 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_2:
            Var_Q = 0b00110000;//2
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("2 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("2 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_3:
            Var_Q = 0b00111000;//3
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("3 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("3 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_4:
            Var_Q = 0b00111100;//4
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("4 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("4 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_5:
            Var_Q = 0b00111110;//5
            Var_TC12 = 1;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("5 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("5 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_6:
            Var_Q = 0b00111111;//6
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("6 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("6 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_7:
            Var_Q = 0b00011111;//7
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("7 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("7 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_8:
            Var_Q = 0b00001111;//8
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("8 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("8 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
		case Num_9:
            Var_Q = 0b00000111;//9
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("9 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("9 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
        case Num_10:
            Var_Q = 0b00000011;//A
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("10 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("10 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;
        case Num_11:
            Var_Q = 0b00000001;//B
            Var_TC12 = 0;
            if ((LCD_Flag == 1)&&(Var_UD_L==1)) {			
                lcd_clear();
                lcd_home();						
                lcd_puts("11 ...");
                lcd_home2();			
                lcd_puts(" ... count up ...");
                }else if((LCD_Flag == 1)&&(Var_UD_L==0)){
                lcd_clear();
                lcd_home();						
                lcd_puts("11 ...");
                lcd_home2();			
                lcd_puts(" ... count down ...");
                }
            LCD_Flag = 0;
			break;    
		default:
			error = 1;	// If no states have been selected, there is an error
						// and the systems must be stopped 
			break;
	}

    write_outputs(); // The interface with the hardware to convert variables 
					 // into voltages at a given microcontroller pin. 
    
	return (error);

}

//***************************************************************************
//Interrupt Service Routine 
//***************************************************************************

static void interrupt ISR (void){	//"interrupt" is the key word here
	GIE = 0; 		//Disable interrupts while attending one of them ...
	if(INT0IF == 1) {		// Is it an INT0 interrupt ?  PHASE 1
	//GIE = 0;		
// If the code execution reach this point is because a falling edge signal 
// in RB0 has been detected, so let's set the flag and the FSM will 
// acknowledge it
        Var_INT0_Flag = 1;	//Set the "software flag"	  
		INT0IF = 0;	// clear the "hardware" INT0 interrupt flag
        //GIE = 1;
	}
    if(INT1IF ==1){         // Is it an INT1 interrupt ?  
       // GIE = 0;
        St_Sp_flag=1;
        INT1IF =0;
        //GIE = 1;
    }
    if(TMR2IF == 1) {			// Is it a Timer2 overflow?
		Counter_var ++; 
		if (Counter_var == Counter_var_Max_Count){
		 	Timer_Flag = 1; 	// Set the flag	to match the external clock
            Counter_var=0;
            if(Count_wave == 2){  //we double the time to give a visual aid in the oscilloscope
                TMR2_wave=0;
                Count_wave =0;
            }else{
                TMR2_wave=1;
                Count_wave++;
            }
            
		}	
		TMR2IF = 0;			// Clear the flag			
	}
	GIE = 1; 		//Enable interrupts 
}


/* =============================================================================
 Scan inputs and process the bits so that we can set our convenient variables 
 many of the variables are "char" because we can easily monitor them using
 the watch window for debugging purposes.
  =========================================================================== */
void read_inputs (void){     

// Like in P9, read the port, mask the bit of interest and set the variable 
    Var_CE = (PORTA & 0b00000001);
    Var_UD_L = (PORTA & 0b00000010)>>1;
    Var_TMR2= (PORTA & 0b00000100)>>2;
}

/* =============================================================================
	Driving the output pins. Let's drive the output ports, converting 
	microcontroller variables into electrical signals.    
   ========================================================================== */
void write_outputs(void){
	
    //To write a given output pin:
    if(Var_TMR2==1){
    PORTC = Var_Q | (Var_TC12 << 6)|(TMR2_wave <<7);    
    }else{
    PORTC = Var_Q | (Var_TC12 << 6);
    }
}


//*****************************************************************************
// Initialise the microcontroller system  
//*****************************************************************************
void init_system (void){
// Configuration bits

// These are the configuration bits translated to the XC8 Compiler
// Microcontroller general configuration bits. You don't have to change them.
// PIC18F4520 Configuration Bit Settings. They can be generated automatically 
// in the MPLAB X IDE

// CONFIG1H
#pragma config OSC = XT       // Oscillator Section bits (XT oscillator)
#pragma config FCMEN = OFF     
// Fail-Safe Clock Monitor E bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       
// Internal/External Osc Switch-over bit (Oscillator Switch-over mode disabled)

// CONFIG2L
#pragma config PWRT = ON       // Power-up Timer E bit (PWRT enabled)
#pragma config BOREN = SBORDIS 
// Brown-out Reset E bits (Brown-out Reset enabled in hardware 
// only (SBOREN is disabled))
#pragma config BORV = 3    // Brown Out Reset Voltage bits (Minimum setting)

// CONFIG2H
#pragma config WDT = OFF
// Watchdog Timer E bit (WDT disabled (control is on the SWDTEN bit))
#pragma config WDTPS = 32768    
// Watchdog Timer Postscale Sect bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC  
// CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF    
// PORTB A/D E bit (PORTB<4:0> pins are configured 
// as digital I/O on Reset)
#pragma config LPT1OSC = OFF   
// Low-Power Timer1 Oscillator E bit (Timer1 configured for 
// higher power operation)
#pragma config MCLRE = ON
// MCLR Pin E bit (MCLR pin enabled; RE3 input pin disabled)
 
// CONFIG4L
#pragma config STVREN = ON      
// Stack Full/Underflow Reset E bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF
// Single-Supply ICSP E bit (Single-Supply ICSP disabled)
#pragma config XINST = OFF
// Extended Instruction Set E bit (Instruction set extension and Indexed 
// Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF
// Code Protection bit (Block 0 (000800-001FFFh) not code-protected)
#pragma config CP1 = OFF        
// Code Protection bit (Block 1 (002000-003FFFh) not code-protected)
#pragma config CP2 = OFF        
// Code Protection bit (Block 2 (004000-005FFFh) not code-protected)
#pragma config CP3 = OFF        
// Code Protection bit (Block 3 (006000-007FFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        
// Boot Block Code Protection bit 
// (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        
// Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       
// Write Protection bit (Block 0 (000800-001FFFh) not write-protected)
#pragma config WRT1 = OFF       
// Write Protection bit (Block 1 (002000-003FFFh) not write-protected)
#pragma config WRT2 = OFF       
// Write Protection bit (Block 2 (004000-005FFFh) not write-protected)

#pragma config WRT3 = OFF       
// Write Protection bit (Block 3 (006000-007FFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       
// Configuration Register Write Protection bit (Configuration registers 
// (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF
// Boot Block Write Protection bit (Boot block (000000-0007FFh) 
// not write-protected)
#pragma config WRTD = OFF
// Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      
// Table Read Protection bit (Block 0 (000800-001FFFh) not protected from 
// table reads executed in other blocks)
#pragma config EBTR1 = OFF      
// Table Read Protection bit (Block 1 (002000-003FFFh) not protected from 
// table reads executed in other blocks)
#pragma config EBTR2 = OFF      
// Table Read Protection bit (Block 2 (004000-005FFFh) not protected from table 
// reads executed in other blocks)
#pragma config EBTR3 = OFF      
// Table Read Protection bit (Block 3 (006000-007FFFh) not protected from table 
// reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      
// Boot Block Table Read Protection bit (Boot block (000000-0007FFh) 
// not protected from table reads executed in other blocks)

    /* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
// Initialise starts here:

/* All the PORTA, PORTB and PORTE is set to be digital I/O (The can be 
     * defined as analogue inputs*/
    ADCON1 	= 0x0F;	// read the datasheet chapter on the A/D converter
// All unused pins are defined as outputs and reset
   
	PORTA 	= 0x00;         // Reset all Flip-Flops at PORTB                
	LATA 	= 0x00;         
	TRISA 	= 0b0000111; 	// PORTB1 and PORTB0 are inputs; PORTB2 is output  
    
    PORTB 	= 0x00;         // Reset all Flip-Flops at PORTB                
	LATB 	= 0x00;         
	TRISB 	= 0b0000011;
    
	PORTC 	= 0x00;         // Reset all Flip-Flops at PORTD
	LATC 	= 0x00;
	TRISC 	= 0b00000000;	// PORTD(7..4) are outputs for the code
    
    PORTD 	= 0x00;         // Reset all Flip-Flops at PORTD
	LATD 	= 0x00;
	TRISD 	= 0b00000000;
                            // the other bits are also defined as outputs    
    
// INT0 edge selection: '0' --> on falling edge 
// (interrupt after clicking the pushbutton)
	INTEDG0	= 0; 
	INT0IE 	= 1; // Enable INT0 interrupts  
	INT1IE  = 1; // Enable INT1 interrupts
    
    T2CKPS0	= 1;	
	T2CKPS1 = 1;	// Prescaler is 16

	TOUTPS0 = 1;
	TOUTPS1	= 1;
	TOUTPS2	= 0;
	TOUTPS3	= 1;	// Postscaler is 12
    
    TMR2ON  = 1;
	TMR2IE 	= 1;	// Timer2 interrupts not allowed when initiating the microcontroller	
					// Activating the Timer2 interrupts will be done in "Set_Timer2" state
    
	IPEN = 0; 	// Interrupt priority disabled

	GIE = 1;		// Global interrupts allowed 
    PEIE 	= 1;	// Enable interrups from other peripherals ( Timer 2 in this example)
	
    Var_CLK_Flag = 0;	// Reset the flag to detect an interrupt INT
    St_Sp_flag  = 0;	// Reset the flag to detect an interrupt INT
    
    lcd_init(FOURBIT_MODE); 	/* Initialise the LCD module  */
	LCD_Flag = 1;
}